#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define BLOCK_SIZE 256
#define MAX_FILENAME_LENGTH 16
#define MAX_BLOCKS_PER_FILE 2040
#define MAX_FILES_IN_DIRECTORY 64
#define FILESYSTEM_SIZE (8 * 1024 * 1024) // 8 Mo
#define SUPERBLOCK_START_BLOCK 1024


void readBlock(unsigned int num, int offset, unsigned char *storage, int size) {
    FILE *file = fopen("filesystem.bin", "rb+"); // Utiliser "rb+" pour ouvrir le fichier en lecture et écriture
    if (file == NULL) {
        perror("Erreur lors de l'ouverture du fichier");
        return;
    }
    fseek(file, num * BLOCK_SIZE + offset, SEEK_SET);
    fread(storage, 1, size, file);
    fclose(file);
}

void writeBlock(unsigned int num, int offset, const unsigned char *storage, int size) {
    FILE *file = fopen("filesystem.bin", "rb+"); // Utiliser "rb+" pour ouvrir le fichier en lecture et écriture
    if (file == NULL) {
        perror("Erreur lors de l'ouverture du fichier");
        return;
    }
    fseek(file, num * BLOCK_SIZE + offset, SEEK_SET);
    fwrite(storage, 1, size, file);
    fclose(file);
}


void LS() {
    unsigned char buffer[MAX_FILENAME_LENGTH];
    int fileCount = 0;

    // Parcourir les blocs multiples de 16 à partir de 0 jusqu'à MAX_FILES_IN_DIRECTORY
    for (int blockNum = 0; blockNum < MAX_FILES_IN_DIRECTORY; blockNum += 16) {
        readBlock(blockNum, 0, buffer, MAX_FILENAME_LENGTH);

        // Vérifier si le nom de fichier est vide
        if (buffer[0] != 0) {
            // Afficher le nom du fichier
            printf("%s\n", buffer);
            fileCount++;
        }
    }

    if (fileCount == 0) {
        printf("Aucun fichier trouvé.\n");
    }
}

void setBlockAvailability(int blockNum, int availability) {
    // Calculer l'index du byte et le bit offset correspondant
    int byteIndex = blockNum / 8;
    int bitOffset = blockNum % 8;

    // Lire le byte existant du bloc de disponibilité des blocs
    unsigned char buffer[1];
    readBlock(SUPERBLOCK_START_BLOCK + byteIndex, 0, buffer, 1);

    // Mettre à jour le bit d'offset correspondant
    if (availability) {
        buffer[0] |= (1 << bitOffset);
    } else {
        buffer[0] &= ~(1 << bitOffset);
    }

    // Écrire le byte mis à jour dans le bloc de disponibilité des blocs
    writeBlock(SUPERBLOCK_START_BLOCK + byteIndex, 0, buffer, 1);
}


void RM(const char *filesystem_path, const char *filename) {
    unsigned char buffer[BLOCK_SIZE];
    int fileNum = -1;

    // Parcourir les blocs réservés pour la description des fichiers (superbloc)
    for (int blockNum = 0; blockNum < MAX_FILES_IN_DIRECTORY; blockNum += 16) {
        unsigned char filenameBuffer[MAX_FILENAME_LENGTH];
        readBlock(blockNum, 0, filenameBuffer, MAX_FILENAME_LENGTH);

        // Vérifier si le bloc contient le nom du fichier recherché
        if (memcmp(filenameBuffer, filename, MAX_FILENAME_LENGTH) == 0) {
            // Effacer le nom du fichier dans le superbloc
            memset(filenameBuffer, 0, MAX_FILENAME_LENGTH);
            writeBlock(blockNum, 0, filenameBuffer, MAX_FILENAME_LENGTH);

            unsigned char fileBuffer[MAX_BLOCKS_PER_FILE * 2];
            readBlock(blockNum, MAX_FILENAME_LENGTH, fileBuffer, MAX_BLOCKS_PER_FILE * 2);

            // Libérer les blocs associés au fichier et réinitialiser les numéros de blocs
            for (int i = 0; i < MAX_BLOCKS_PER_FILE * 2; i += 2) {
                int blockNum = ((int)fileBuffer[i] << 8) + (int)fileBuffer[i + 1];
                if (blockNum == 0) {
                    break; // Fin du fichier
                }
                setBlockAvailability(blockNum, 1); // Marquer le bloc comme disponible
                fileBuffer[i] = 0;
                fileBuffer[i + 1] = 0;
            }

            // Mettre à jour les numéros de blocs associés au fichier
            writeBlock(blockNum, MAX_FILENAME_LENGTH, fileBuffer, MAX_BLOCKS_PER_FILE * 2);

            fileNum = blockNum;
            break; // Fichier trouvé, sortir de la boucle
        }
    }

    // Afficher le résultat de l'opération
    if (fileNum == -1) {
        printf("Le fichier \"%s\" n'a pas été trouvé.\n", filename);
    } else {
        printf("Le fichier \"%s\" a été supprimé avec succès.\n", filename);
    }
}



        
// Fonction pour créer un fichier avec le nom donné et le contenu fourni en entrée standard
//pour l'instant la fonction ajoute seulement le nom du fichier dans le superblock
void TYPE(const char *filesystem_path, const char *filename) {

    unsigned char buffer[MAX_FILENAME_LENGTH];

    // Parcours des blocs réservés pour la description des fichiers (superbloc)
    for (int blockNum = 0; blockNum <= MAX_FILES_IN_DIRECTORY; blockNum += 16) {
        readBlock(blockNum, 0, buffer, MAX_FILENAME_LENGTH);

        // Vérifier si le bloc est vide (pas de nom de fichier)
        if (buffer[0] == 0) {
            // Écrire le nom du fichier dans l'emplacement vide du superbloc
            writeBlock(blockNum, 0, (const unsigned char *)filename, MAX_FILENAME_LENGTH);

            break; // Fichier créé, sortir de la boucle
        }
    }

    printf("Le fichier \"%s\" a été créé avec succès.\n", filename);
}




// Fonction pour afficher le contenu d'un fichier
void CAT(const char *filesystem_path, const char *filename) {

}




int main(int argc, char *argv[]) {
    // Vérification du nombre d'arguments
    if (argc < 3) {
        printf("Usage: %s filesystem_path ACTION [arguments...]\n", argv[0]);
        printf("Actions:\n");
        printf("  LS\n");
        printf("  TYPE filename\n");
        printf("  CAT filename\n");
        printf("  RM filename\n"); // Ajout de l'action RM
        return 1;
    }

    // Récupération des arguments
    const char *filesystem_path = argv[1];
    const char *action = argv[2];

    // Exécution de l'action correspondante en fonction des arguments passés
    if (strcmp(action, "LS") == 0) {
        LS(filesystem_path);
    } else if (strcmp(action, "TYPE") == 0) {
        if (argc < 4) {
            printf("Usage: %s filesystem_path TYPE filename\n", argv[0]);
            return 1;
        }
        TYPE(filesystem_path, argv[3]);
    } else if (strcmp(action, "CAT") == 0) {
        if (argc < 4) {
            printf("Usage: %s filesystem_path CAT filename\n", argv[0]);
            return 1;
        }
        CAT(filesystem_path, argv[3]);
    } else if (strcmp(action, "RM") == 0) { // Ajout de l'action RM
        if (argc < 4) {
            printf("Usage: %s filesystem_path RM filename\n", argv[0]);
            return 1;
        }
        RM(filesystem_path, argv[3]);
    } else {
        printf("Action non reconnue. Actions possibles : LS, TYPE, CAT, RM.\n");
        return 1;
    }

    return 0;
}

